# flake8: noqa
# type: ignore

# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import warnings

import briton_pb2 as briton__pb2
import grpc

GRPC_GENERATED_VERSION = "1.63.0"
GRPC_VERSION = grpc.__version__
EXPECTED_ERROR_RELEASE = "1.65.0"
SCHEDULED_RELEASE_DATE = "June 25, 2024"
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower

    _version_not_supported = first_version_is_lower(
        GRPC_VERSION, GRPC_GENERATED_VERSION
    )
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    warnings.warn(
        f"The grpc package installed is at version {GRPC_VERSION},"
        + f" but the generated code in briton_pb2_grpc.py depends on"
        + f" grpcio>={GRPC_GENERATED_VERSION}."
        + f" Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}"
        + f" or downgrade your generated code using grpcio-tools<={GRPC_VERSION}."
        + f" This warning will become an error in {EXPECTED_ERROR_RELEASE},"
        + f" scheduled for release on {SCHEDULED_RELEASE_DATE}.",
        RuntimeWarning,
    )


class BritonStub(object):
    """The greeting service definition."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Infer = channel.unary_stream(
            "/briton.Briton/Infer",
            request_serializer=briton__pb2.InferenceRequest.SerializeToString,
            response_deserializer=briton__pb2.InferenceAnswerPart.FromString,
            _registered_method=True,
        )


class BritonServicer(object):
    """The greeting service definition."""

    def Infer(self, request, context):
        """Sends a greeting"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_BritonServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Infer": grpc.unary_stream_rpc_method_handler(
            servicer.Infer,
            request_deserializer=briton__pb2.InferenceRequest.FromString,
            response_serializer=briton__pb2.InferenceAnswerPart.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "briton.Briton", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class Briton(object):
    """The greeting service definition."""

    @staticmethod
    def Infer(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/briton.Briton/Infer",
            briton__pb2.InferenceRequest.SerializeToString,
            briton__pb2.InferenceAnswerPart.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )
